culvertSim <- function(startYear, endYear, climate, strategy, cost, costList, 
                       paramList = list(loc = 171.6821, scale = 56.868, shape = -.0123), rStart = T, antForce = F) {
  # Simulates the preloaded culvert list through replacements and ext events
  # 
  # Args:
  #   startYear, endYear: start and end of simulation, determines how many years the sim runs for
  #   climate: determines the amount of climate change must be "high, medium, low, normal, random"
  #   strategy: adaptation strategy to be used must be "anticipate, EE, normal, upgrade"
  #   cost: empty cost data.frame to be filled in and returned
  #   costList: empty list equal to startYear - endYear + 1 to be filled in with a cost
  #     data.frame for each year of the simulation
  #   paramList: list of parameters to be used for extreme events GEV current parameters are based
  #     on data from colorado
  #   rStart: T/F if T install years are randomally determied in function, if false the passed install
  #     are used
  #   antForce: if T forces installs for all culverts in the first nrow(culvertList) years of the simulation
  #     if F install years are shortened by the amount specified using anticipateAmount
  #
  # Returns: the costList, which is a list of cost data.frames with startYear - endYear + 1
  #     elements
  #
  
  # Number of years the simulation runs for
  numYears <- endYear - startYear + 1
  
  # Randomly generate installation dates for culverts if rStart = T
  # Currently this can only be done with lifespans of 50 and 80 years, distributions
  # would need to be developed for additional years
  if(rStart){
    for(i in 1:nrow(culvertList)){ # Runs for each 
      if (culvertList$serviceLife[i] == 80){
        sampleFrame <- eightyYearSL
      }else{
        sampleFrame <- fiftyYearSL
      }
      culvertList$installYear[i] <- sample(0:(sample(sampleFrame, 1) - 1), 1) * -1
    }
  }else{
    culvertList$installYear <- startYears
  }
  
  # What to do if the anticipate strat is being used
  # for true culvert replacement is forced in the first n years
  # for false culvert replacement is accelerated by reducing
  # service life by anticipate amount (as a percent of orig service life)
  
  if(strategy == "anticipate"){
    if(antForce){
      forceInstall <- sample(1:nrow(culvertList), nrow(culvertList), replace = F)
    }else{
      culvertList$installYear <- culvertList$installYear - culvertList$serviceLife * anticipateAmount
      forceInstall <- rep(F, nrow(culvertList))
    }
  }else{
    forceInstall <- rep(F, nrow(culvertList))
  }
  
  # Get new location parameters for the gev distribution based on climate change
  locations <- getLocation(climate, paramList, culvertList$designFlood, numYears)
  
  # Generate random extreme events for n culverts through n years using location parameters
  # from above. 
  events <- matrix(data = rgev(numYears*nrow(culvertList), loc = locations, 
                               scale = paramList$scale, shape = paramList$shape), 
                   ncol = nrow(culvertList), nrow = numYears, byrow = T)
  
  # Iterate through years of replacements and EE----------------------------------
  for(i in startYear:endYear){
    
    # Determine size of design event for each crossing
    # this is based on the orignal GEV parameters if the crossing has been upgraded
    # the design flood in the culvert list will reflect it
    designEvent <- qevd(1-1/(culvertList$designFlood), paramList$loc, paramList$scale, paramList$shape)
    
    # Iterate through culvert list and replace/record damage as necessary
    for(j in 1:nrow(culvertList)){
      
      # retrieve road and culvert for current iteratn
      currentRoad <- roadList[roadList$roadName == culvertList$crossingRoad[j],]
      currentCulvert <- culvertList[j,]
      
      # Capacity is exceeded-----------------------------------------------------------
      if(events[i,j] > designEvent[j]) {
        
        # Determine event amount as a percent of capacity
        exceedAmount <- (events[i,j] - designEvent[j]) / designEvent[j]
        
        # Call eventProcessor
        cost[j,] <- eventProcessor(currentCulvert, currentRoad, exceedAmount, currentYear, strategy, 
                                   paramList, culvertUpgraded)
        
        # If the current strategy is EE and a the crossing was replaced after an event
        # change the strategy to upgrade
        if(cost$replacement[j] == 1 & strategy == "EE"){
          strategy <- "upgrade"
        }
        
        # Replace the crossing if damage was sufficient
        if(cost$replacement[j] == 1){
          
          # Determine new culvert info based on strategy
          replaceInfo <- culvertLife(currentCulvert, currentRoad, strategy, currentYear, T)
          
          # Replace culvert with potentially updated design flood
          culvert <- replaceInfo$newCulvert
          culvertList <- replaceCulvert(culvert$culvertName, culvertList, currentYear, sameDesign = F, 
                                        designFlood = culvert$designFlood, material = culvert$material, 
                                        length = culvert$length, diameter = culvert$diameter, 
                                        serviceLife = culvert$serviceLife, replaceDelay = culvert$replaceDelay, 
                                        cost = culvert$cost, culvertCostFT = culvert$culvertCostFT)
          
          # Mark the culvert as upgraded
          culvertUpgraded$upgraded[j] <- T
          currentCulvert <- culvertList[j,]
        }
        
      # Capacity was not exceeded---------------------------------
      } else { 
        
        # Update cost data.frame
        cost$culvertName[j] <- currentCulvert$culvertName
        cost$floodDamage[j] <- 0
        cost$year <- currentYear
        cost$iteration <- iterationNum
        
        # End of Service life replacements------------------------------------------------
        
        # Force replacement for anticipate strategy with a force
        if(currentCulvert$designFlood == permCulvertList$designFlood[j] &
           strategy == "anticipate" & currentYear == forceInstall[j]){
          replaceInfo <- culvertLife(currentCulvert, currentRoad, strategy, currentYear, T)
        
        # Check for replacement
        }else{
          replaceInfo <- culvertLife(currentCulvert, currentRoad, strategy, currentYear)
        }
        
        # Update culvert list if crossing was replaced
        if(replaceInfo$TFreplace){
          culvert <- replaceInfo$newCulvert
          culvertList <- replaceCulvert(culvert$culvertName, culvertList, currentYear, sameDesign = F, 
                                        designFlood = culvert$designFlood, material = culvert$material, 
                                        length = culvert$length, diameter = culvert$diameter, 
                                        serviceLife = culvert$serviceLife, replaceDelay = culvert$replaceDelay, 
                                        cost = culvert$cost, culvertCostFT = culvert$culvertCostFT)
          
          # Update Cost List
          cost$construction[j] <- culvertCost(culvert, currentRoad, paramList, culvertUpgraded, emergency = F)
          cost$delayHours[j] <- costOfDelay(currentRoad, currentCulvert$replaceDelay, planned = T)
          cost$replacement[j] <- 1
          
          # If crossing was upgraded record it in the upgrade list
          
          if(culvertList$designFlood[j] > permCulvertList$designFlood[j]){
            culvertUpgraded$upgraded[j] <- T
          }
        
        # if crossing was not replaced up date cost data.frame  
          }else{
            cost$replacement[j] <- 0
            cost$delayHours[j] <- 0
            cost$construction[j] <- 0
          }
      } # End of else for no damaging events
    } # End for loop for culverts
    
    # Assign the current year's cost to the i'th element of the costList
    costList[[i]] <- cost
    
    # Increase the current Year by one
    currentYear <- currentYear + 1
    
  } # End for loop for years
  
  # Resotre culvert list for the next iteration and reset currentYear
  currentYear <- 1
  iterationNum <<- iterationNum + 1
  culvertList <<- permCulvertList
  culvertUpgraded$upgraded <<- F
  return(costList)
}


#****************************************************#
#****Run the culvert simulation with random chars****#
#****************************************************#
culvertSimR <- function(startYear, endYear, climate, strategy, cost, costList, 
                       paramList = list(loc = 171.6821, scale = 56.868, shape = -.0123), rStart = T) {
  
  numYears <- endYear - startYear + 1
  ##Randomly generate installation dates for culverts
#   if(rStart){
#     for(i in 1:nrow(culvertList)){
#       if(culvertList$serviceLife[i] == 80){
#         sampleFrame <- eightyYearSL
#       }else{
#         sampleFrame <- fiftyYearSL
#       }
#       culvertList$installYear[i] <- sample(0:(sample(sampleFrame, 1) - 1), 1) * -1
#     }
#   }else{
  # }
  #culvertList <<- culvertList
  #print(culvertList)
  initialValue <- ((culvertList$installYear + culvertList$serviceLife)/culvertList$serviceLife) * culvertList$cost
  currentYear <- 1
  changeYear <- 0
  if(strategy == "anticipate"){
    forceInstall <- sample(1:nrow(culvertList), nrow(culvertList), replace = F)
  }else{
    forceInstall <- rep(F, nrow(culvertList))
  }
  
  locations <- getLocation(climate, paramList, culvertList$designFlood, numYears)
  events <- matrix(data = rgev(numYears*nrow(culvertList), loc = locations, 
                        scale = paramList$scale, shape = paramList$shape), 
                   ncol = nrow(culvertList), nrow = numYears, byrow = T)

  ##Iterate through the years##
  for(i in startYear:endYear){

    designEvent <- qevd(1-1/(culvertList$designFlood), paramList$loc, paramList$scale, paramList$shape)
    
    ####Runs simultation once for each culvert######
    ################################################
    
    ##Iterate over culvertList...This could potentially be vectorized...
    for(j in 1:nrow(culvertList)) {
      
      #retrieve road and culvert for current iteratn
      currentRoad <- roadList[roadList$roadName == culvertList$crossingRoad[j],]
      currentCulvert <- culvertList[j,]
      
      #Determines the number of events which exceed the culverts design capacity
      
      ###Call the event processor if an event occured####
      ###################################################
      if(events[i,j] > designEvent[j]) {
        exceedAmount <- (events[i,j] - designEvent[j]) / designEvent[j]
        cost[j,] <- eventProcessorR(currentCulvert, currentRoad, exceedAmount, currentYear, strategy, 
                                   paramList, culvertUpgraded)
        
        if(cost$replacement[j] == 1 & strategy == "EE"){
          strategy <- "upgrade"
        }
        if(cost$replacement[j] == 1){
          changeYear <- currentYear
          replaceInfo <- culvertLife(currentCulvert, currentRoad, strategy, currentYear, T)
          culvert <- replaceInfo$newCulvert
          culvertList <- replaceCulvert(culvert$culvertName, culvertList, currentYear, sameDesign = F, 
                                        designFlood = culvert$designFlood, material = culvert$material, 
                                        length = culvert$length, diameter = culvert$diameter, 
                                        serviceLife = culvert$serviceLife, replaceDelay = culvert$replaceDelay, 
                                        cost = culvert$cost, culvertCostFT = culvert$culvertCostFT)
          culvertUpgraded$upgraded[j] <- T
          currentCulvert <- culvertList[j,]
        }
        
        ###Fill in the cost list when no event occured###  
        #################################################
      } else { 
        cost$culvertName[j] <- currentCulvert$culvertName
        cost$floodDamage[j] <- 0
        cost$year <- currentYear
        cost$iteration <- iterationNum
        
        ###Determine if culvert needs to be replaced###
        ###############################################
        
        #There is the potential for a culvert to be damaged, repaired, and replaced in the same year though this is extremely unlikely
        if(currentCulvert$designFlood == permCulvertList$designFlood[j] &
           strategy == "anticipate" & currentYear == forceInstall[j]){
          replaceInfo <- culvertLife(currentCulvert, currentRoad, strategy, currentYear, T)
        }else{
          replaceInfo <- culvertLife(currentCulvert, currentRoad, strategy, currentYear)
        }
        if(replaceInfo$TFreplace){
          #print(paste("replacing", currentCulvert$culvertName, "installed", currentCulvert$installYear,
                      #"in year:", currentYear))
          culvert <- replaceInfo$newCulvert
          
          culvertList <- replaceCulvert(culvert$culvertName, culvertList, currentYear, sameDesign = F, 
                                        designFlood = culvert$designFlood, material = culvert$material, 
                                        length = culvert$length, diameter = culvert$diameter, 
                                        serviceLife = culvert$serviceLife, replaceDelay = culvert$replaceDelay, 
                                        cost = culvert$cost, culvertCostFT = culvert$culvertCostFT)
          
          cost$construction[j] <- culvertCost(culvert, currentRoad, paramList, culvertUpgraded, emergency = F)
          cost$delayHours[j] <- costOfDelay(currentRoad, currentCulvert$replaceDelay, planned = T)
          if(culvertList$designFlood[j] > permCulvertList$designFlood[j])culvertUpgraded$upgraded[j] <- T
          cost$replacement[j] <- 1
          #print(culvertList)
          
        }else{
          cost$replacement[j] <- 0
          cost$delayHours[j] <- 0
          cost$construction[j] <- 0
        }
      } #End of else for no damaging events
    } #End for loop for culverts
    
    #Assign the current year's cost to the i'th element of the costList
    costList[[i]] <- cost
    #Increase the current Year by one
    currentYear <- currentYear + 1
  } #End for loop for years
  
  ##Resotre culvert list for the next iteration and reset currentYear
  currentValue <- sapply(1:nrow(culvertList), function(x){
    myCulvert <- culvertList[x,]
    myRoad <- roadList[roadList$roadName == myCulvert$crossingRoad,]
    return(culvertCost(myCulvert, myRoad, paramList, culvertUpgraded, F))
  })
  
  finalValue <- ((culvertList$serviceLife - (currentYear - culvertList$installYear))/culvertList$serviceLife) * culvertList$cost - initialValue
  
  currentYear <- 1
  #print(culvertList)
  iterationNum <<- iterationNum + 1
  #culvertList <<- permCulvertList
  culvertUpgraded$upgraded <<- F
  returnList <- list(changeYear, costList, finalValue)
  return(returnList)
}



#*******************************************************#
#**Return culvert Cost**********************************#
#*******************************************************#
culvertCost <- function(culvert, road, paramList, culvertUpgraded, emergency = F) {
  
  # Determines cost based on a percent increase of original flow, whether the replacement
  # is an emergency and if the crossing has previously been upgraded
  # 
  # Args:
  #   culvert: one row from the culvertList
  #   road: one row from the roadList
  #   paramList: named list of parameters for a GEV must include loc, scale, and shape
  #   culvertUpgraded: a culvertUpgrade data.frame
  #   emergency: T/F is the crossing being replaced after an EE
  # Returns: a numeric value for the cost of replacement
  #
  
  # Create variable for base culvert cost
  totalCost <- culvert$cost
  
  # Retrived the information for the original culvert from the locked list
  origCulvert <- permCulvertList[permCulvertList$culvertName == culvert$culvertName,]
  
  # Determine if the crossing being installed has a higher capacity than the original crossing
  if(culvert$designFlood > origCulvert$designFlood){
    
    # Calculate the capacity of the orignal crossing and the one being installed
    origFlowDesign <- qevd(1-(1/origCulvert$designFlood), paramList$loc, paramList$scale, paramList$shape)
    newFlowDesign <- qevd(1-(1/culvert$designFlood), paramList$loc, paramList$scale, paramList$shape)
    
    # Percent increase of new crossing over original
    increaseAmount <- (newFlowDesign - origFlowDesign)/origFlowDesign
    
    # Base increased in cost of upgraded culvert based on the upgradeCost parameter
    # and how much the crossing's capacity was increased
    increaseCost <- totalCost * increaseAmount * upgradeCost
    
    # New total cost if this is the first time the crossing is being upgraded
    if(!culvertUpgraded$upgraded[culvertUpgraded$culvertName == culvert$culvertName]){
      totalCost <- totalCost + increaseCost
    
    # Total cost if the crossing has previously been upgraded
    }else{
      totalCost <- totalCost + (increaseCost * postUpgradeFactor)
    } 
    
  } 
  
  # Adjust total cost if the crossing is being replaced after an Ext Event
  if(emergency){
    totalCost <- totalCost * emergencyFactor
    }
  return(totalCost)
}


#*******************************************************#
#*event processor with random chars*********************#
#*******************************************************#

eventProcessorR <- function(culvert, road, exceedeAmount, currentYear, strategy, paramList, culvertUpgraded) {
  
  ##Create a cost frame and assign current state variables
  tempCost <- data.frame(matrix(ncol = 7, nrow = 1))
  colnames(tempCost) <- c("culvertName", "delayHours", "floodDamage", "construction", "year", "replacement", "iteration")
  tempCost$year <- currentYear
  tempCost$iteration <- iterationNum
  tempCost$culvertName <- culvert$culvertName
  
  ##This is pretty janky but it produces a distribution so about 20% of events wipe out a 100 yr culvert
  eventseverity <- (exceedeAmount/maxDamage) * .5

  ##Calulate Culvert Damage, at 1 the damage is considered to be 50% of orginal Value
  
  emergencyCost <- culvertCost(culvert, road, paramList, culvertUpgraded, emergency = T)
  normalCost <- culvertCost(culvert, road, paramList, culvertUpgraded, emergency = F)
  
  culvertDamage <- eventseverity * normalCost
  culvertValue <- normalCost * (1 - ((currentYear - culvert$installYear) / culvert$serviceLife))
   
  
  #print(paste("name:", culvert$culvertName, "year:", currentYear, "damage:", culvertDamage, "value:", culvertValue, "severity:", eventseverity))
  #if event severity is less than 1 the crossing is not destroyed and cost are as follows
  if(culvertDamage < (culvertValue)){
    ##calculate cost of delay
    ##delayDays are a first order estimate to be improved
    delayDays <- rtriangle(1,.1,3,.6)
    delayCost <- costOfDelay(road,delayDays,planned = T)
    if(delayCost>5000000){delayCost <- 5000000}
    tempCost$delayHours <- delayCost
    
    #*****************Cost of repair****************************************************#
    #This is done based on flood severity and orginal cost of the culvert***************#
    #It is assumed that at 50% of culvert cost the crossing is considered failed and****# 
    #is replaced so event severity 1 is the point at which the culvert is replaced******#
    #This is converted to .5 and everthing else is converted appropiately***************#
    #***********************************************************************************#
    
    ##Assign damages, construction is assumed to be zero
    tempCost$floodDamage <- culvertDamage
    tempCost$construction <- 0
    tempCost$replacement <- 0
    
    #Entered if eventseverity > 1 and crossing is destroyed apprx 20% of the time  
  } else { 
    
    #determine delay days, in this configuration the weibull resembles an extreme value distribution, the distribution is truncated at 1 and 40
    #These numbers are based off of Perrin et al loosely.
    delayDays <- rtriangle(1, 1, 4, 2)
    delayCost <- costOfDelay(road,delayDays,planned = F) + costOfDelay(road, culvert$replaceDelay, planned = T)
    tempCost$delayHours <- delayCost
    
    #Calculate the cost of constructing a new culvert


      
    ##Calculate the cost to replace culvert based on strategy and current culvert
    if(strategy == "normal" | culvert$designFlood > permCulvertList$designFlood[permCulvertList$culvertName == culvert$culvertName]){
      construction <- normalCost
      floodDamage <- emergencyCost - normalCost + culvertValue
    } else {
      
      culvert$designFlood <- culvert$designFlood * upgradeFactor
      newCostE <- culvertCost(culvert, road, paramList, culvertUpgraded, emergency = T)
      newCostN <- culvertCost(culvert, road, paramList, culvertUpgraded, emergency = F)
      floodDamage <- newCostE - newCostN + culvertValue
      construction <- newCostN
    }
    tempCost$replacement <- 1
    tempCost$floodDamage <- floodDamage
    tempCost$construction <- construction
    
    #Return the DF temp cost which should include cost for the two events for that culvert for that year
  } #End else containing culvert destroyed
  
  return(tempCost)
} #End Function


#*******************************************************#
#*Determine consequences of a flood event***************#
#*******************************************************#

eventProcessor <- function(culvert, road, exceedeAmount, currentYear, strategy, paramList, culvertUpgraded) {
  
  ##Create a cost frame and assign current state variables
  tempCost <- data.frame(matrix(ncol = 7, nrow = 1))
  colnames(tempCost) <- c("culvertName", "delayHours", "floodDamage", "construction", "year", "replacement", "iteration")
  tempCost$year <- currentYear
  tempCost$iteration <- iterationNum
  tempCost$culvertName <- culvert$culvertName
  
  eventseverity <- (exceedeAmount/maxDamage) * .5
  
  ##Calulate Culvert Damage, at 1 the damage is considered to be 50% of orginal Value
  culvertDamage <- eventseverity * culvertCost(culvert, road, paramList, culvertUpgraded, emergency = T)
  culvertValue <- culvertCost(culvert, road, paramList, culvertUpgraded, emergency = F) * (1 - ((currentYear - culvert$installYear) / culvert$serviceLife))
  culvertValue <- ifelse(culvertValue < 0, 0, culvertValue)
  
  #if event severity is less than 1 the crossing is not destroyed and cost are as follows
  if(culvertDamage < (culvertValue)){
    ##calculate cost of delay
    ##delayDays are a first order estimate to be improved
    delayDays <- rtriangle(1,.1,3,.6)
    delayCost <- costOfDelay(road,delayDays,planned = T)
    if(delayCost>5000000){delayCost <- 5000000}
    tempCost$delayHours <- delayCost
    
    #*****************Cost of repair****************************************************#
    #This is done based on flood severity and orginal cost of the culvert***************#
    #It is assumed that at 50% of culvert cost the crossing is considered failed and****# 
    #is replaced so event severity 1 is the point at which the culvert is replaced******#
    #This is converted to .5 and everthing else is converted appropiately***************#
    #***********************************************************************************#
    
    ##Assign damages, construction is assumed to be zero
    tempCost$floodDamage <- culvertDamage
    tempCost$construction <- 0
    tempCost$replacement <- 0
    
    #Entered if eventseverity > 1 and crossing is destroyed apprx 20% of the time  
  } else { 
    
    #determine delay days, in this configuration the weibull resembles an extreme value distribution, the distribution is truncated at 1 and 40
    #These numbers are based off of Perrin et al loosely.
    delayDays <- rtriangle(1, 1, 4, 2)
    delayCost <- costOfDelay(road,delayDays,planned = F) + costOfDelay(road, culvert$replaceDelay, planned = T)
    tempCost$delayHours <- delayCost
    
    #Calculate the cost of constructing a new culvert
    tempCost$construction <- 0
    
    ##Calculate the cost to replace culvert based on strategy and current culvert
    if(strategy == "normal" | culvert$designFlood > permCulvertList$designFlood[permCulvertList$culvertName == culvert$culvertName]){
      floodDamage <- culvertCost(culvert, road, paramList, culvertUpgraded, emergency = T)
    } else {
      culvert$designFlood <- culvert$designFlood * upgradeFactor
      floodDamage <- (culvertCost(culvert, road, paramList, culvertUpgraded, emergency = T))
    }
    tempCost$replacement <- 1
    tempCost$floodDamage <- floodDamage
    
    #Return the DF temp cost which should include cost for the two events for that culvert for that year
  } #End else containing culvert destroyed
  
  return(tempCost)
} #End Function


#*******************************************************#
#**Replaces culvert in culvert list*********************#
#*******************************************************#
replaceCulvert <- function(culvertName, culvertList, currentYear, sameDesign = T, designFlood = NA, material = NA,
                           length = NA, diameter = NA, replaceDelay = NA, serviceLife = NA, cost = NA, 
                           culvertCostFT = NA){
  
  ##Copy culvert from list and remove it
  currentCulvert <- culvertList[culvertList$culvertName == culvertName,]
  culvertList <- culvertList[!culvertList$culvertName == culvertName,]
  
  ##Check to see if  new design parameters are specified
  if(sameDesign) {
    culvertList <- createCulvert(culvertName, currentCulvert$crossingRoad, currentYear, currentCulvert$designFlood, currentCulvert$material, currentCulvert$length, 
                                 currentCulvert$diameter, currentCulvert$serviceLife, currentCulvert$replaceDelay, 
                                 currentCulvert$cost, currentCulvert$culvertCostFT, culvertList)
  } else {
    culvertList <- createCulvert(culvertName, currentCulvert$crossingRoad, currentYear, designFlood, material, length, 
                                 diameter, serviceLife, replaceDelay, cost, culvertCostFT, culvertList)
  }
  
  ##Sort and return culvert list with new culvert added
  culvertList <- culvertList[order(culvertList$culvertName),]
  rownames(culvertList) <- NULL
  return(culvertList)
}


#*******************************************************#
#**Cost of delay based on Perrin et al.*****************#
#*******************************************************#
costOfDelay <- function(road, days, planned = T){
  rateOfPersonDelay <- 17.18
  rateofTruckDelay <- 50
  if(planned) {
    delayTime <- road$delayPlan
  } else {
    delayTime <- road$delayFail
  }
  cost <- road$AADT*delayTime*days*((1-road$percentTruck)*road$vehicleOccu*rateOfPersonDelay +
                                      rateofTruckDelay*road$percentTruck)
  return(cost)
}


#*******************************************************#
#**Creates a new road object****************************#
#*******************************************************#
createRoad <- function(roadName, AADT, delayPlan, delayFail, vehicleOccu, percentTruck, lanes, growth, roadList = NA) {
  ##Create Road
  myRoad <- data.frame(roadName, AADT, delayPlan, delayFail, vehicleOccu, percentTruck, lanes, growth, stringsAsFactors = F)
  
  ##If a roadlist was past add new road to road list
  suppressWarnings( if (!(is.na(roadList))){
    myRoad <- rbind(roadList, myRoad)
  } )
  return(myRoad)
}


#**********************************************#
#***Returns a vector of climate factors********#
#**********************************************#
getClimate <- function(climateTrend, numberYears) {
  climateTrend <- tolower(climateTrend)

  ##A normal trend returns a stationary climate##
  percentYears = min(c(1, numberYears/100))
  
  if(climateTrend == "normal") {
    riskFactor <- 1
    
    ##A high trend returns a climate that is tripling the chance of an event over 100 years##
  } else if(climateTrend == "high") {
    riskFactor <- highFactor
    
    ##Medium risk factor is a doubling of the chance of an event over 100 years##
  } else if(climateTrend == "medium") {
    riskFactor <- medFactor
    
    ##Low risk factor is an increas of 1.5x over 100 years##  
  } else if(climateTrend == "low") {
    riskFactor <- lowFactor
  } else if(climateTrent == "random") {
    riskFactor <- randomFactor
  }
  
  ##Calculate the change in risk each year assuming a linear trend
  endRisk <- percentYears * riskFactor
  climateFactor <- c(seq(1, endRisk, length.out = 100),rep(endRisk, max(numberYears - 100, 0)))
  
  ##Assuming variation around risk is normally distributed
  climateMean = 0 
  climateSD = .1 
  climateVariaence <- rnorm(numberYears, mean = climateMean, sd = climateSD)
  return(climateFactor <- climateFactor + climateVariaence)
}

#**********************************************#
#***Returns location based on trend and year***#
#**********************************************#
getLocation <- function(climateTrend, paramList, designValue, numYears) {
  climateTrend <- tolower(climateTrend)
  
  ##A normal trend returns a stationary climate##
  
  riskStart <- qgev(1-1/(designValue), paramList$loc, paramList$scale, paramList$shape, paramList$shape)
  
  if(climateTrend == "normal") {
    
    yearlyRisk <- riskStart
    
    ##A high trend returns a climate that is tripling the chance of an event over 100 years##
  } else if(climateTrend == "high") {
    
    yearlyRisk <- qgev(1-1/(designValue * highFactor), paramList$loc, paramList$scale, paramList$shape)
    
    ##Medium risk factor is a doubling of the chance of an event over 100 years##
  } else if(climateTrend == "medium") {
    
    yearlyRisk <- qevd(1-1/(designValue * medFactor), paramList$loc, paramList$scale, paramList$shape)
    
    ##Low risk factor is an increas of 1.5x over 100 years##  
  } else if(climateTrend == "low") {
    
    yearlyRisk <- qevd(1-1/(designValue * lowFactor), paramList$loc, paramList$scale, paramList$shape)
    
  } else if(climateTrend == "random"){
    yearlyRisk <- qevd(1-1/(designValue * randomFactor), paramList$loc, paramList$scale, paramList$shape)
  }
  locations <- matrix(NA, nrow = length(designValue), ncol = numYears)
  for(i in 1:length(designValue)){
    locations[i,1:100] <- seq(paramList$loc, (yearlyRisk[i] - riskStart[i]) + paramList$loc, length.out = 100)
  }
  #locations[,101:numYears] <- locations[,100]
  
  
  ##Calculate the change in risk each year assuming a linear trend
  
    ##Assuming variation around risk is normally distributed
  
  return(locations) #+ climateVariaence)
}

#**********************************************#
#***Determines if culvert needs to be replaced*#
#**and returns cost of replacement based on ***#
#*strategy*************************************#
#**********************************************#

culvertLife <- function(culvert, road, strategy, currentYear, forceReplace = F){
  replaceInfo <- vector("list", 2)
  names(replaceInfo) <- c("TFreplace", "newCulvert")
  if(currentYear - culvert$installYear >= culvert$serviceLife | forceReplace){
    replaceInfo$TFreplace <- T
    if(strategy == "normal" | strategy == "EE" | culvert$designFlood > permCulvertList$designFlood[permCulvertList$culvertName == culvert$culvertName]) {
      
      replaceInfo$newCulvert <- culvert
    }
    
    else{
      culvert$designFlood <- culvert$designFlood * upgradeFactor
      replaceInfo$newCulvert <- culvert
    }
    
  }else{
    replaceInfo$TFreplace <- F
  }
  return(replaceInfo)
}

#****************************************************#
#Create culvertList or add culvert to list***********#
#****************************************************#
createCulvert <- function(culvertName, crossingRoad, installYear, designFlood, material, length, diameter, 
                          serviceLife, replaceDelay, cost, culvertCostFT, culvertList = NA) {
  
  #Ensure culvert is being created on a known road
  if(!any(roadList$roadName == crossingRoad)) {stop("please add road before creating culvert")}
  
  #Create a culvert object based on the arguments passed to the function
  myCulvert <- data.frame(culvertName, crossingRoad, installYear, designFlood, material, length, diameter, 
                          serviceLife, replaceDelay, cost, culvertCostFT, stringsAsFactors = F)
  
  #Add the culvert created above to the data frame of culverts if one exist
  suppressWarnings( if (!(is.na(culvertList))){
    if(any(culvertList$culvertName == culvertName)) {stop("Culvert Name Must be Unique")}
    myCulvert <- rbind(culvertList, myCulvert)
  } )
  
  #return either single culvert or data frame of culverts
  return(myCulvert)
}



##***************************************##
#function to be called by snow fall******##
#****************************************##
snowWrapper <- function(a){
  #climate <- "High"
  #strategy <- "EE"
  costFrame <- culvertSim(startYear, endYear, climate, strategy, cost, costList, antForce = F)
  costFrame <- rbindlist(costFrame)
  return(costFrame)
}

##***************************************##
#snowfall w/o rand start*****************##
#****************************************##
snowWrapperL <- function(a){
  #climate <- "High"
  #strategy <- "EE"
  costFrame <- culvertSim(startYear, endYear, climate, strategy, cost, costList, rStart = F)
  costFrame <- rbindlist(costFrame)
  return(costFrame)
}

##****************************************************************************##
#snowfall function for bigCulvert generation, aggregates runs prior to return*##
#*****************************************************************************##
snowWrapperR <- function(a){

  myResult <- culvertSimR(startYear, endYear, climate, strategy, cost, costList, rStart = F)
  costFrame <- createCostDF(myResult[[2]])
  currentListCul <- cbind(doByCulvert(costFrame), as.integer(myResult[[3]]))
  setnames(currentListCul, "V2", "SUM_infraValue")
#   costFrame <- doByStatFrame(costFrame)
#   costFrame[, emergencyF := emergencyFactor]
#   costFrame[, upgradeC := upgradeCost]
#   costFrame[, upgradeF := upgradeFactor]
#   costFrame[, climateF := randomFactor]
#   costFrame[, maxD := maxDamage]
#   costFrame[, averageStartAge := postUpgradeFactor]
#   costFrame <- melt(costFrame, id.vars = c(1, 11:16))
#   costFrame[, strategy := strategy]
  
  currentListCul[, emergencyF := emergencyFactor]
  currentListCul[, upgradeC := upgradeCost]
  currentListCul[, upgradeF := upgradeFactor]
  currentListCul[, climateF := randomFactor]
  currentListCul[, maxD := maxDamage]
  currentListCul[, changeYear := as.integer(myResult[[1]])]
  currentListCul <- melt(currentListCul, id.vars = c(1, 2, 13:18))
  currentListCul[, strategy := strategy]
#   returnList <- list(costFrame, currentListCul)
#   names(returnList) <- c("statList", "culvertStat")
  
  ###Test Functions
#   returnList <- list(currentListCul, costFrame)
#   names(returnList) <- c("culvertStat", "costFrame")
#   return(returnList)
  return(currentListCul)
}

##********************************************************************************##
#*Used to generate large list of random parameters without rerunning any params***##
#*********************************************************************************##
snowWrapperR2 <- function(a){
  
  startYears <- vector("numeric", length = nrow(culvertList))
  for(i in 1:8){
    if(culvertList$serviceLife[i] == 80){
      sampleFrame <- 59
    }else{
      sampleFrame <- 49
    }
    startYears[i] <- sample(1:sampleFrame, 1) * -1
  }
  emergencyFactor <<- runif(1, emergencyRange[1], emergencyRange[2])
  maxDamage <<- runif(1, maxDamageRange[1], maxDamageRange[2])
  #postUpgradeFactor <- runif(1, postUpgradeFactorRange[1], postUpgradeFactorRange[2])
  upgradeCost <<- runif(1, upgradeCostRange[1], upgradeCostRange[2])
  upgradeFactor <<- runif(1, upgradeFactorRange[1], upgradeFactorRange[2])
  randomFactor <<- runif(1, 1, 3)
  culvertList$installYear <<- startYears
  
  stratResults <- vector("list", length(strats))
  jCount <- 0
  for(j in strats){
    jCount <- jCount + 1
    myResult <- culvertSimR(startYear, endYear, climate, j, cost, costList, rStart = F)
    costFrame <- createCostDF(myResult[[2]])
    currentListCul <- cbind(doByCulvert(costFrame), as.integer(myResult[[3]]))
    setnames(currentListCul, "V2", "SUM_infraValue")
    
    currentListCul[, changeYear := as.integer(myResult[[1]])]
    currentListCul <- melt(currentListCul, id.vars = c(1, 2, 13))
    currentListCul[, strategy := j]
    stratResults[[jCount]] <- currentListCul 
  }
  currentListCul <- data.table(data.frame(lapply(stratResults, function(x) x[, value])))
  setnames(currentListCul, strats)
  currentListCul <- cbind(currentListCul, stratResults[[1]][,c(1,2,4), with = F])
  currentListCul[, emergencyF := emergencyFactor]
  currentListCul[, upgradeC := upgradeCost]
  currentListCul[, upgradeF := upgradeFactor]
  currentListCul[, climateF := randomFactor]
  currentListCul[, maxD := maxDamage]
  
}

##****************************************************************************##
#Light and fast routine returning minimal data*##
#*****************************************************************************##
snowWrapperR_lite <- function(a){
  
  myResult <- culvertSimR(startYear, endYear, climate, strategy, cost, costList, rStart = F)
  costFrame <- createCostDF(myResult[[2]])
  currentListCul <- doByCulvert(costFrame, slim = T)
  
  currentListCul[, emergencyF := emergencyFactor]
  currentListCul[, upgradeC := upgradeCost]
  currentListCul[, upgradeF := upgradeFactor]
  currentListCul[, climateF := randomFactor]
  currentListCul[, maxD := maxDamage]
  currentListCul <- melt(currentListCul, id.vars = c(1, 2, 5:9))
  currentListCul[, strategy := strategy]
  return(currentListCul)
}


#****************************************************#
#convert list to df with total columns***************#
#****************************************************#
createCostDF <- function(costList){
  costDF <- rbindlist(costList)
  costDF[,totalCost := floodDamage + construction, which = F]
  return(costDF)
}

doByCulvert <- function(costDF, slim = F){
  if(!slim){
    sd.cols <- c("construction", "delayHours", "floodDamage", "totalCost")
    myNames <- c("construction", "delayHours", "floodDamage", "totalCost")
  }else{
    sd.cols <- c("totalCost")
    myNames <- "totalCost"
  }
  npvFrame <- costDF[, lapply(.SD, function(x) as.integer(npv(discount, c(0,x)))), by = .(iteration, culvertName), .SDcols = sd.cols]
  sumFrame <- costDF[, lapply(.SD, function(x) as.integer(sum(x))), by = .(iteration, culvertName), .SDcols = c(sd.cols, "replacement")]
  setnames(npvFrame, c("Iteration", "culvertName", paste0("NPV_", myNames)))
  
  sumFrame[, c("iteration", "culvertName"):=NULL,]
  setnames(sumFrame, paste0("SUM_", c(myNames, "replacement")))
  if(slim){
    sumFrame[, "SUM_replacement" := NULL]
  }
  returnFrame <- cbind(npvFrame, sumFrame)
}

doByStatFrame <- function(costDF){
  ######################################################
  ########Create DF with Sum and NPV Info###############
  ######################################################
  
  #####Calculates sum and NPV for entire system**********
  #******************************************************
  sd.cols <- c("construction", "delayHours", "floodDamage", "totalCost")
  
  npvFrame <- costDF[, lapply(.SD, function(x) npv(discount, c(0,x))), by = .(iteration), .SDcols = sd.cols]
  sumFrame <- costDF[, lapply(.SD,sum), by = .(iteration), .SDcols = c(sd.cols, "replacement")]
  setnames(npvFrame, c("Iteration", "NPV_construction", "NPV_delayHours", "NPV_floodDamage", 
                       "NPV_totalCost"))
  
  sumFrame[, c("iteration"):=NULL,]
  setnames(sumFrame, c("SUM_construction", "SUM_delayHours", "SUM_floodDamage", 
                       "SUM_totalCost", "SUM_replacements"))
  returnFrame <- cbind(npvFrame, sumFrame)
}

doByYear <- function(costDF){
  sumFrame <- summaryBy(construction + delayHours + floodDamage + totalCost + replacement~
                          iteration + year, data = costDF, FUN = sum)
  setnames(sumFrame, c("Iteration", "year", "SUM_construction", "SUM_delayHours", "SUM_floodDamage", 
                          "SUM_totalCost", "SUM_replacements"))
  
  return(sumFrame)
}
